import CodeMirror from 'codemirror';
import 'codemirror/mode/gfm/gfm.js';
import 'codemirror/addon/display/placeholder.js';
import { MarkdownEditorOptionsComplete, MarkdownEditorOptions, FromTextareaOptionsComplete, MdeFromTextareaOptions, MarkdownEditorShortcuts } from './markdown-editor-options';
declare class MarkdownEditorBase {
    protected static readonly ORDERED_LIST_PATTERN: RegExp;
    protected static readonly UNORDERED_LIST_PATTERN: RegExp;
    protected static readonly CHECK_LIST_PATTERN: RegExp;
    protected static readonly BOLD_TOKENS: string[];
    protected static readonly ITALIC_TOKENS: string[];
    readonly cm: CodeMirror.Editor;
    protected options: MarkdownEditorOptionsComplete;
    constructor(codemirror: CodeMirror.Editor, options: MarkdownEditorOptionsComplete);
    /***** Basic Editor API *****/
    /**
     * Toggle "bold" for each selection.
     */
    toggleBold(): void;
    /**
     * Toggle "italic" for each selection.
     */
    toggleItalic(): void;
    /**
     * Toggle "strikethrough" for each selection.
     */
    toggleStrikethrough(): void;
    /**
     * Toggle "inline code" for each selection.
     */
    toggleInlineCode(): void;
    /**
     * Toggle inline formatting for each selection by wrapping it with the specified token
     * and unwrapping it with the specified token or one of the alternative tokens.
     * @param token the token
     * @param altTokens the alternative tokens
     */
    protected toggleInlineFormatting(token: string, altTokens?: string[], preventDoubledTokenMatch?: boolean): void;
    /**
     * Set the specified heading level for each selected line. If `level` is 0, the heading token is removed.
     * @param level the heading level
     */
    setHeadingLevel(level: 0 | 1 | 2 | 3 | 4 | 5 | 6): void;
    /**
     * Increase the heading level for each selected line, i.e. make the heading smaller.
     * If the current heading level is at the maximum of 6, the heading token is just removed.
     * If the current heading level is 0 (no heading), the new heading level is set to 1.
     */
    increaseHeadingLevel(): void;
    /**
     * Decrease the heading level for each selected line, i.e. make the heading bigger.
     * If the current heading level is at the minimum of 1, the heading token is just removed.
     * If the current heading level is 0 (no heading), the new heading level is set to 6.
     */
    decreaseHeadingLevel(): void;
    /**
     * Toggle "quote" for each selected line.
     */
    toggleQuote(): void;
    /**
     * Toggle "unordered list" for each selected line. Furthermore, a selected ordered list line is
     * transformed to an unordered list.
     */
    toggleUnorderedList(): void;
    /**
     * Toggle "ordered list" for each selected line. Furthermore, a selected unordered list line is
     * transformed to an ordered list. Additionally adjusts the subsequent lines that are connected
     * to the list of the selected line.
     */
    toggleOrderedList(): void;
    /**
     * Adjust the enumeration of subsequent lines in same ordered list as line *baseLineNumber*.
     * @param baseLineNumber the selected line which is toggled
     * @param baseListNumber the list number of the selected line (should be 0, if list starts after selected line)
     */
    protected processNextLinesOfOrderedList(baseLineNumber: number, baseListNumber: number): void;
    /**
     * Toggle "check list" for each selected line. Furthermore, a selected (un)ordered list line is
     * transformed to a check list.
     */
    toggleCheckList(): void;
    /**
     * Replace each selected line with the result of the callback function `replaceFn`.
     * Additionally adjusts the selection boundaries to the originally selected boundaries.
     * @param replaceFn callback function to the calculate the line replacements
     */
    protected replaceTokenAtLineStart(replaceFn: (oldLineContent: string, lineNumber: number) => string): void;
    /**
     * Wrap each selection with code block tokens, which are inserted in separate lines.
     */
    insertCodeBlock(): void;
    /**
     * Wrap a link template around each selection.
     */
    insertLink(): void;
    /**
     * Wrap a image link template around each selection.
     */
    insertImageLink(): void;
    /**
     * Wrap a template around each selection with the specified `before` and `after` template parts.
     * @param before the template part inserted **before** the selection start
     * @param after the template part inserted **after** the selection start
     */
    protected insertInlineTemplate(before: string, after: string): void;
    /**
     * Insert a horizontal rule in the subsequent line of each selection.
     */
    insertHorizontalRule(): void;
    /**
     * Insert a table template with the specified number of rows and columns
     * in the subsequent line of each selection.
     * @param rows number of rows
     * @param columns number columns
     */
    insertTable(rows?: number, columns?: number): void;
    /**
     * Insert a block template in the subsequent line of each selection.
     * The template can contain multiple lines separated with the specified `lineSeparator`.
     * @param template the template
     * @param lineSeparator The line separator. Default is `\n`.
     */
    protected insertBlockTemplateBelow(template: string, lineSeparator?: string): void;
    /**
     * Open a Markdown Guide in a new tab.
     */
    openMarkdownGuide(): void;
    /***** Extended Editor API *****/
    /**
     * Undo one edit. Shortcut for `Codemirror.undo()`.
     */
    undo(): void;
    /**
     * Redo one edit. Shortcut for `Codemirror.redo()`.
     */
    redo(): void;
    /**
     * Toggle the editor's rich-text mode. If off, there is no markdown styling inside the editor.
     */
    toggleRichTextMode(): void;
    /**
     * Start a file download containing the editor content as plain text.
     * The name of the file is either `fileName` if specified, or the default `downloadFileName`
     * specified in the Markdown Editor options.
     * @param fileName The name of the downloaded file. Preferred over `options.downloadFileName`.
     */
    downloadAsFile(fileName?: string): void;
    /**
     * Import content from a file. Reads the content of `file` if specified, or opens the browser's
     * file selection dialog and reads the content of the user-selected file.
     * @param file The file to import the content from.
     */
    importFromFile(file?: File): void;
    /**
     * Formats the content using Prettier's Markdown parser.
     */
    formatContent(): void;
    /***** Developer API *****/
    /**
     * Get the editor's content with the specified line break format.
     * @param lineSeparator The line break format. Default is `\n`.
     */
    getContent(lineSeparator?: string): string;
    /**
     * Get the editor's content with the lines as array.
     */
    getContentPerLine(): string[];
    /**
     * Set the editor's content with the specified line break format.
     * @param content The content.
     * @param lineSeparator The line break format. Default is `\n`.
     */
    setContent(content: string, lineSeparator?: string): void;
    /**
     * Get the editor's content with the lines as array.
     */
    setContentPerLine(content: string[]): void;
    /**
     * Focus the editor. Shortcut for `Codemirror.focus()`.
     */
    focus(): void;
    /**
     * Get the number of characters in the document.
     */
    getCharacterCount(): number;
    /**
     * Get the number of words in the document.
     */
    getWordCount(): number;
    /**
     * Get the current cursor position as a `{line, ch}` object.
     * Shortcut for `Codemirror.getCursor()`.
     */
    getCursorPos(): CodeMirror.Position;
    /**
     * Returns whether the document has been modified.
     * Inverted shortcut for `Codemirror.isClean()`
     */
    isDirty(): boolean;
    /**
     * Check if text at current cursor position has specified token type
     * like returned by `CodeMirror.getTokenTypeAt()`.
     * @param token
     */
    hasTokenAtCursorPos(token: string): boolean;
    /**
     * Get the type of list in the specified line. Return undefined, if the line contains no list token.
     * @param lineNumber
     */
    getListTypeOfLine(lineNumber: number): 'unordered' | 'ordered' | 'check' | undefined;
    /**
     * Get the action shortcuts effectively applied to the editor.
     *
     * This might be a mix of custom shortcuts
     * and default shortcuts as specified in `DEFAULT_OPTIONS`.
     */
    getShortcuts(): MarkdownEditorShortcuts;
    /**
     * Add the specified keyboard shortcut to the specified action function.
     *
     * **Note:** This does **not** replace an existing shortcut to the specified action,
     * but only **adds** it. In case of an already existing shortcut, the result will be
     * two distinct shortcuts to the same function. However, you can use `removeShortcut()`
     * to remove the old one.
     * @param hotkeys the hotkeys string as required by Codemirror
     * @param action the action function
     * @see https://codemirror.net/doc/manual.html#keymaps
     */
    addShortcut(hotkeys: string, action: () => void): void;
    /**
     * Remove the specified keyboard shortcut.
     *
     * @param hotkeys the hotkeys string as required by Codemirror
     * @see https://codemirror.net/doc/manual.html#keymaps
     */
    removeShortcut(hotkeys: string): void;
    /***** Markdown Editor Options *****/
    /**
     * Overwrites current options with specified options.
     * Options that are not included in specified `options` will not be modified.
     * @param options the set of options that shall be changed
     */
    setOptions(options: MarkdownEditorOptions | undefined): void;
    /**
     * Apply codemirror-specific options that are specified in `this.options`.
     */
    protected applyCodemirrorOptions(): void;
    /**
     * Apply the key map for markdown editor actions specified in `this.options.shortcuts`
     * to the Codemirror editor instance.
     */
    protected applyEditorKeyMappings(): void;
    /***** Private methods *****/
    /**
     * Create the GFM mode object. It includes some adjustments for the default
     * theme to use Codemirror's default markup styling while always providing
     * expressive class names for the tokens.
     */
    private getGfmMode;
    /**
     * Remove the `.cm-link` class from images' alt texts because text is not a link.
     * This has been hardcoded to Codemirror's Markdown mode and thus cannot be changed
     * without compiling Codemirror ourselves within this project, which is not the
     * desired way so far.
     */
    private removeLinkClassFromImageTexts;
}
export declare class MarkdownEditor extends MarkdownEditorBase {
    constructor(element: HTMLElement, options?: MarkdownEditorOptions);
}
export declare class MarkdownEditorFromTextarea extends MarkdownEditorBase {
    readonly cm: CodeMirror.EditorFromTextArea;
    protected options: FromTextareaOptionsComplete;
    protected saver?: (instance: CodeMirror.Editor) => void;
    constructor(textarea: HTMLTextAreaElement, options?: MdeFromTextareaOptions);
    /**
     * Copy the content of the editor into the textarea.
     *
     * Shortcut for `Codemirror.EditorFromTextarea.save()`.
     */
    syncTextarea(): void;
    /**
     * Remove the editor, and restore the original textarea (with the editor's current content).
     *
     * Shortcut for `Codemirror.EditorFromTextarea.toTextarea()`.
     */
    toTextarea(): void;
    /**
     * Returns the textarea that the instance was based on.
     *
     * Shortcut for `Codemirror.EditorFromTextarea.getTextarea()`.
     */
    getTextarea(): HTMLTextAreaElement;
    /**
     * @inheritdoc
     */
    setOptions(options: MarkdownEditorOptions | undefined): void;
}
export {};
//# sourceMappingURL=markdown-editor.d.ts.map